<?php

/**
 * EyeTV AutoTuner Report
 *
 * Reads the EyeTV AutoTuner log and a station map file and displays
 * the results of the auto-tuning process.
 *
 * To use this script, place it and the station map somewhere in
 * the Sites folder of the user who ran the auto-tuner.  The script
 * will look for the log file in that user's Library folder.
 *
 * @author Marshall Elfstrand <marshall@vengefulcow.com>
 */

// Path to EyeTV autotuner log file inside user home directory.
// An absolute path can be specified by starting the path with a slash ("/").
define("LOG_FILE", "Library/EyeTV Archive/EyeTVAutoTune.log");

// Name of station map file.
// The station map is a CSV-formatted file with the following fields:
//   actualCh,logicalCh,callsign,network,city,direction,distance,digital
define("STATION_FILE", "stations.csv");

// Thresholds for signal levels.
// 0% is "noSignal", and anything below DECENT_THRESHOLD is "poor".
define("DECENT_THRESHOLD", 40);
define("GOOD_THRESHOLD", 65);
define("GREAT_THRESHOLD", 90);

// Pattern to look for in log entries.
define("LOG_PATTERN",
       '/.* Tuning to ([0-9]*): (Signal|No Signal) found. ' .
       'Signal Quality=([0-9]*)%\. ([0-9]*) Services found/');

// Determine what we're supposed to display (all/digital/analog).
$filter = strtolower($_REQUEST["show"]);
if ($filter != "digital" && $filter != "analog") $filter="all";

// Build the table of filter links.
// Don't link the currently selected filter.
$filterLinks["all"] = "All Channels";
$filterLinks["digital"] = "Digital Only";
$filterLinks["analog"] = "Analog Only";
foreach ($filterLinks as $key=>$value) {
    if ($key != $filter) {
        $filterLinks[$key] =
            '<a href="report.php?show=' . $key . '">' . $value . '</a>';
    }
}

// Read the station map.
$stations = array();
$stationFile = fopen(STATION_FILE, "r");
while (($fields = fgetcsv($stationFile, 500)) !== FALSE) {
    $newStation = array();
    $newStation["actualChannel"] = $fields[0];
    $newStation["logicalChannel"] = $fields[1];
    $newStation["callsign"] = $fields[2];
    $newStation["network"] = $fields[3];
    $newStation["city"] = ucwords(strtolower($fields[4]));
    $newStation["direction"] = $fields[5];
    $newStation["distance"] = $fields[6];
    $newStation["isDigital"] = (strtolower($fields[7]) == "true");
    $stations[$newStation["actualChannel"]] = $newStation;
}
fclose($stationFile);

// Determine the location of the log file.
$logPath = LOG_FILE;
if ($logPath[0] != "/") {
    // Path is relative to home directory.  Determine full path.
    $thisPath = $_SERVER["SCRIPT_FILENAME"];
    $logPath =
        substr($thisPath, 0, strpos($thisPath, "/Sites/") + 1) . LOG_FILE;
}

// Parse the entries in the log file.
$results = array();
$lines = file($logPath);
foreach ($lines as $line) {
    $matches = array();
    if (preg_match(LOG_PATTERN, $line, $matches)) {
    
        // Extract the fields from the log entry.
        $result = array();
        $result["frequency"] = $matches[1] / 1000000;  // store in MHz
        $result["hasSignal"] = ($matches[2] == "Signal");
        $result["strength"] = $matches[3];
        $result["serviceCount"] = $matches[4];
        
        // Determine the channel number based on the frequency.
        if ($result["frequency"] < 72) {
            // Lower VHF Station
            $result["actualChannel"] =
                (string) floor(($result["frequency"] - 42) / 6);
        }
        elseif ($result["frequency"] < 174) {
            // Middle VHF Station
            $result["actualChannel"] =
                (string) floor(($result["frequency"] - 46) / 6);
        }
        elseif ($result["frequency"] < 470) {
            // Upper VHF Station
            $result["actualChannel"] =
                (string) floor(($result["frequency"] - 132) / 6);
        }
        else {
            // UHF Station
            $result["actualChannel"] =
                (string) floor(($result["frequency"] - 386) / 6);
        }
        
        // Fill in the values from the station map if possible.
        if (array_key_exists($result["actualChannel"], $stations)) {
            $station = $stations[$result["actualChannel"]];
            $result["logicalChannel"] = $station["logicalChannel"];
            $result["callsign"] = $station["callsign"];
            $result["description"] = $station["network"] . ", " .
                $station["city"] . ", " . $station["direction"] . ", " .
                $station["distance"] . " miles away";
            if ($station["isDigital"]) $result["isDigital"] = true;
        }

        // Determine what style to give the station, based on
        // signal strength and type.
        if ($result["isDigital"])
            $suffix="-digital";
        else
            $suffix = "-analog";
        if ($result["strength"] == 0)
            $result["style"] = "noSignal";
        elseif ($result["strength"] < DECENT_THRESHOLD)
            $result["style"] = "poor" . $suffix;
        elseif ($result["strength"] < GOOD_THRESHOLD)
            $result["style"] = "decent" . $suffix;
        elseif ($result["strength"] < GREAT_THRESHOLD)
            $result["style"] = "good" . $suffix;
        else
            $result["style"] = "great" . $suffix;
        
        // Add asterisk if the station provides a recognized service.
        if ($result["serviceCount"] > 0) $result["hasServices"] = "*";
            
        // Add the station to the list if it matches the filter.
        if ( ($filter == "all") ||
             ($result["isDigital"] && $filter == "digital") ||
             (!$result["isDigital"] && $filter == "analog")
        ) $results[] = $result;
    }
}

?>

<html>
  <head>
    <title>EyeTV Auto-Tuner Results</title>
    <style>
      h1 {
        background: #004080;
        color: #FFFFFF;
        font-family: "Lucida Grande", Arial, Helvetica, sans-serif;
        font-size: 18px;
        padding: 9px;
      }
      table {
        font-family: "Lucida Grande", Arial, Helvetica, sans-serif;
        font-size: 13px;
        margin: 18px;
      }
      th {
        background: #000000;
        color: #FFFFFF;
        padding: 4px;
        text-align: left;
      }
      td {
        text-align: right;
      }
      .noSignal {
        color: #C0C0C0;
      }
      .poor-analog {
        color: #808080;
      }
      .poor-digital {
        background: #C0C0C0;
        color: #FFFFFF;
      }
      .decent-analog {
        color: #800000;
      }
      .decent-digital {
        background: #804040;
        color: #FFFFFF;
      }
      .good-analog {
        color: #808000;
      }
      .good-digital {
        background: #808000;
        color: #FFFFFF;
      }
      .great-analog {
        color: #008000;
      }
      .great-digital {
        background: #008000;
        color: #FFFFFF;
      }
      .textField {
        text-align: left;
      }
      .hasServices {
        background: #FFFFFF;
        color: #000000;
        font-weight: bold;
        text-align: center;
      }
    </style>
  </head>
  <body>
    <h1>EyeTV Auto-Tuner Results</h1>
    <table>
      <tr>
        <td>[<?php echo $filterLinks["all"] ?>]</td>
        <td>[<?php echo $filterLinks["digital"] ?>]</td>
        <td>[<?php echo $filterLinks["analog"] ?>]</td>
      </tr>
    </table>
    <table>
      <tr>
        <th>Frequency</th>
        <th>Strength</th>
        <th>Actual</th>
        <th>Logical</th>
        <th>Callsign</th>
        <th>Description</th>
        <th>Found</th>
      </tr>
      <?php foreach ($results as $result): ?>
      <tr class="<?php echo $result["style"] ?>">
        <td><?php echo $result["frequency"]      ?> MHz</td>
        <td><?php echo $result["strength"]       ?> %</td>
        <td><?php echo $result["actualChannel"]  ?></td>
        <td><?php echo $result["logicalChannel"] ?></td>
        <td class="textField"><?php echo $result["callsign"] ?></td>
        <td class="textField"><?php echo $result["description"] ?></td>
        <td class="hasServices"><?php echo $result["hasServices"] ?></td>
      </tr>
      <?php endforeach; ?>
    </table>
  </body>
</html>
